#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "pico/stdlib.h"
#include "hardware/i2c.h"
#include "hardware/watchdog.h"
#include "common.h"
#include "DS3231.h"
#include "extern.h"

// DS3231s SRAM memories UTC time
// When read DS3231s change to localtime by using time.h library
void DS3231_init(struct repeating_timer *timer1, struct repeating_timer *timer2)
{
    // I2C Initialisation. Using it at 1Mhz.
    i2c_init(I2C_PORT, 1000 * 1000);
    gpio_set_function(I2C_SDA, GPIO_FUNC_I2C);
    gpio_set_function(I2C_SCL, GPIO_FUNC_I2C);
    gpio_pull_up(I2C_SDA);
    gpio_pull_up(I2C_SCL);

    add_repeating_timer_ms(-200, alarm_ms_callback, NULL, timer1);
    // watchdog
    watchdog_enable(500, 1);
    watchdog_update();
    add_repeating_timer_ms(-10, alarm_xfade_callback, NULL, timer2);

    // enable 1pps
    uint8_t buf[2] = {0};
    buf[0] = DS3231_CONTROL_ADDR;
    i2c_write_blocking(I2C_PORT, DS3231_DEVICE_ADDR, &buf[0], 1, true);
    i2c_read_blocking(I2C_PORT, DS3231_DEVICE_ADDR, &buf[1], sizeof(buf[1]), false);
    buf[1] = buf[1] & ~DS3231_CONTROL_INTCN; // INTCN "0" is 1Hz enable.
    i2c_write_blocking(I2C_PORT, DS3231_DEVICE_ADDR, buf, sizeof(buf), false);
}

void DS3231_read_all(void)
{
    uint8_t buf[DS3231_STATUS_ADDR + 1] = {0};

    // Write start address(DS3231_SEC_ADDR=0x00)
    i2c_write_blocking(I2C_PORT, DS3231_DEVICE_ADDR, DS3231_SEC_ADDR, 1, true);
    // repeat read. 0x00 to 0x0f
    i2c_read_blocking(I2C_PORT, DS3231_DEVICE_ADDR, buf, sizeof(buf), false);

    ds3231.sec = buf[0];
    ds3231.min = buf[1];
    ds3231.hour = buf[2];
    ds3231.day = buf[3];
    ds3231.date = buf[4];
    ds3231.mon = buf[5];
    ds3231.year = buf[6];
    ds3231.AL1_sec = buf[7];
    ds3231.AL1_min = buf[8];
    ds3231.AL1_hour = buf[9];
    ds3231.AL1_date = buf[10];
    ds3231.AL2_min = buf[11];
    ds3231.AL2_hour = buf[12];
    ds3231.AL2_date = buf[13];
    ds3231.control = buf[14];
    ds3231.status = buf[15];

    // generate localtime from UTC
    time_t time;
    struct tm t = {};
    struct tm *new_time;
    t.tm_sec = ((ds3231.sec & 0b01110000) >> 4) * 10 + (ds3231.sec & 0b00001111);
    t.tm_min = ((ds3231.min & 0b01110000) >> 4) * 10 + (ds3231.min & 0b00001111);
    t.tm_hour = ((ds3231.hour & 0b00110000) >> 4) * 10 + (ds3231.hour & 0b00001111);
    t.tm_mday = ((ds3231.date & 0b00110000) >> 4) * 10 + (ds3231.date & 0b00001111);
    t.tm_mon = ((ds3231.mon & 0b00010000) >> 4) * 10 + (ds3231.mon & 0b00001111) - 1;
    t.tm_year = 2000 + ((ds3231.year & 0b11110000) >> 4) * 10 + (ds3231.year & 0b00001111) - 1900;
    time = mktime(&t) + 3600 * setting.timezone;
    new_time = localtime(&time);

    // return localtime to global value
    uint8_t time_buf[2] = {0};
    time_buf[0] = new_time->tm_sec / 10;
    time_buf[1] = new_time->tm_sec - time_buf[0] * 10;
    ds3231.sec = (time_buf[0] << 4) | time_buf[1];
    time_buf[0] = new_time->tm_min / 10;
    time_buf[1] = new_time->tm_min - time_buf[0] * 10;
    ds3231.min = (time_buf[0] << 4) | time_buf[1];
    time_buf[0] = new_time->tm_hour / 10;
    time_buf[1] = new_time->tm_hour - time_buf[0] * 10;
    ds3231.hour = (time_buf[0] << 4) | time_buf[1];
    ds3231.day = new_time->tm_wday;
    time_buf[0] = new_time->tm_mday / 10;
    time_buf[1] = new_time->tm_mday - time_buf[0] * 10;
    ds3231.date = (time_buf[0] << 4) | time_buf[1];
    time_buf[0] = (new_time->tm_mon + 1) / 10;
    time_buf[1] = (new_time->tm_mon + 1) - time_buf[0] * 10;
    ds3231.mon = (time_buf[0] << 4) | time_buf[1];
    time_buf[0] = (new_time->tm_year + 1900 - 2000) / 10;
    time_buf[1] = (new_time->tm_year + 1900 - 2000) - time_buf[0] * 10;
    ds3231.year = (time_buf[0] << 4) | time_buf[1];

    // anti poisoning
    if(ds3231.hour == setting.anti_poisoning_time[0] && ds3231.min == setting.anti_poisoning_time[1] && ds3231.sec == setting.anti_poisoning_time[2] && setting.use_anti_poisoning == true)
    {
        anti_poisoning = true;
    }

#ifndef NDEBUG
    printf("%d/%d/%d/%d %d:%d:%d\n", new_time->tm_year+1900, new_time->tm_mon+1, new_time->tm_mday, new_time->tm_wday, new_time->tm_hour, new_time->tm_min, new_time->tm_sec);
#endif
}

// for sync dot
void DS3231_read_sec(void)
{
    uint8_t buf[1] = {0};

    // Write start address(DS3231_SEC_ADDR=0x00)
    i2c_write_blocking(I2C_PORT, DS3231_DEVICE_ADDR, DS3231_SEC_ADDR, 1, true);
    // repeat read. 0x00 to 0x0f
    i2c_read_blocking(I2C_PORT, DS3231_DEVICE_ADDR, buf, sizeof(buf), false);

    ds3231.sec = buf[0];
}

//DS3231_write_time(gps_time[4], gps_time[5], gps_time[2], gps_time[3], gps_time[0], gps_time[1]);
void DS3231_write_time(uint8_t sec1, uint8_t sec2, uint8_t min1, uint8_t min2, uint8_t hour1, uint8_t hour2)
{
    
    uint8_t buf[4] = {0};

    buf[0] = DS3231_SEC_ADDR;
    buf[1] = (sec1 << 4) | sec2;
    buf[2] = (min1 << 4) | min2;
    buf[3] = (hour1 << 4) | hour2;

    i2c_write_blocking(I2C_PORT, DS3231_DEVICE_ADDR, buf, sizeof(buf), false);
}

// write decimal time data
void DS3231_write_time_decimal(uint8_t hour, uint8_t min, uint8_t sec)
{
    uint8_t buf[4] = {0};
    int8_t temp_hour;
    uint8_t temp;

    buf[0] = DS3231_SEC_ADDR;
    // adjust timezone to UTC
    // user will input loaltime
    temp_hour = hour;
    temp_hour -= setting.timezone;
    if(temp_hour >= 24)
    {
        temp_hour -= 24;
    }
    else if(temp_hour < 0)
    {
        temp_hour += 24;
    }
    hour = (uint8_t)temp_hour;

    temp = sec / 10;
    buf[1] = (temp << 4) | (sec - temp * 10);
    temp = min / 10;
    buf[2] = (temp << 4) | (min - temp * 10);
    temp = hour / 10;
    buf[3] = (temp << 4) | (hour - temp * 10);

    i2c_write_blocking(I2C_PORT, DS3231_DEVICE_ADDR, buf, sizeof(buf), false);
}

//DS3231_write_date(gps_date[0], gps_date[1], gps_date[2], gps_date[3], gps_date[4], gps_date[5]);
void DS3231_write_date(uint8_t date1, uint8_t date2, uint8_t month1, uint8_t month2, uint8_t year1, uint8_t year2)
{
    // not write day
    uint8_t buf[4] = {0};

    buf[0] = DS3231_DATE_ADDR;
    buf[1] = (date1 << 4) | date2;
    buf[2] = (month1 << 4) | month2;
    buf[3] = (year1 << 4) | year2;

    i2c_write_blocking(I2C_PORT, DS3231_DEVICE_ADDR, buf, sizeof(buf), false);
}

// write decimal date data
void DS3231_write_date_decimal(uint8_t year, uint8_t mon, uint8_t date)
{
    // not write day
    uint8_t buf[4] = {0};
    uint8_t temp;

    buf[0] = DS3231_DATE_ADDR;
    temp = date / 10;
    buf[1] = (temp << 4) | (date - temp * 10);
    temp = mon / 10;
    buf[2] = (temp << 4) | (mon - temp * 10);
    temp = year / 10;
    buf[3] = (temp << 4) | (year- temp * 10);

    i2c_write_blocking(I2C_PORT, DS3231_DEVICE_ADDR, buf, sizeof(buf), false);
}